package cpu

import (
	"fmt"

	"git.akyoto.dev/cli/q/src/errors"
	"git.akyoto.dev/cli/q/src/register"
)

// Register represents a single CPU register.
type Register struct {
	ID   register.ID
	user fmt.Stringer
}

// Use marks the register as used by the given object.
func (register *Register) Use(obj fmt.Stringer) error {
	if register.user != nil {
		return &errors.RegisterInUse{Register: register.ID.String(), User: register.user.String()}
	}

	register.user = obj
	return nil
}

// Free frees the register so that it can be used for new calculations.
func (register *Register) Free() {
	register.user = nil
}

// IsFree returns true if the register is not in use.
func (register *Register) IsFree() bool {
	return register.user == nil
}

// String returns a human-readable representation of the register.
func (register *Register) String() string {
	return fmt.Sprintf("%s%s%v", register.ID, "=", register.user)
}
